@@ -1,16 +1,20 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright © Meta Platforms, Inc. and affiliates
 
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 # coding=utf8
+# Can remove once we're >= 3.7 so asyncio has a .run() method
+# pyre-ignore-all-errors[0]
+# pyre-ignore-all-errors[16]
+# pyre-ignore-all-errors[21]
 
 import argparse
 import ast
 import asyncio
 import logging
 import sys
-from collections import defaultdict, namedtuple
+from collections import defaultdict
 from configparser import ConfigParser
 from enum import Enum
 from json import dump
@@ -22,11 +26,27 @@ from shutil import rmtree
 from subprocess import CalledProcessError
 from tempfile import gettempdir
 from time import time
-from typing import Any, Dict, List, Optional, Sequence, Set, Tuple, Union
+from typing import Any, Dict, List, NamedTuple, Optional, Sequence, Set, Tuple, Union
+
+# Support pyproject.toml
+# In >= 3.11 we can remove this import dance
+if sys.version_info >= (3, 11):  # pragma: no cover
+    try:
+        import tomllib
+    except ImportError:
+        # Help users on older alphas
+        import tomli as tomllib
+else:
+    import tomli as tomllib  # type: ignore
 
 
 LOG = logging.getLogger(__name__)
 MACOSX = system() == "Darwin"
+# To make main use asyncio.run and unittests to test approrpiately for older cpython
+# Using 3.8 rather than 3.7 due to subprocess.exec in < 3.8 only support main loop
+# https://bugs.python.org/issue35621
+PY_38_OR_GREATER = sys.version_info >= (3, 8)
+PYPROJECT_TOML = "pyproject.toml"
 WINDOWS = system() == "Windows"
 # Windows needs to use a ProactorEventLoop for subprocesses
 # Need to use sys.platform for mypy to understand
@@ -58,7 +78,7 @@ def _config_read(
     if cp is None:
         cp = _config_default()
 
-    cwd_path = Path(cwd)  # type: Path
+    cwd_path = Path(cwd)
     root_path = Path(f"{cwd_path.drive}\\") if WINDOWS else Path("/")
 
     while cwd_path:
@@ -99,13 +119,27 @@ class StepName(Enum):
     pyre_run = 9
 
 
-coverage_line = namedtuple("coverage_line", ["stmts", "miss", "cover", "missing"])
-step = namedtuple(
-    "step", ["step_name", "run_condition", "cmds", "log_message", "timeout"]
-)
-test_result = namedtuple(
-    "test_result", ["setup_py_path", "returncode", "output", "runtime", "timeout"]
-)
+class coverage_line(NamedTuple):
+    stmts: float
+    miss: float
+    cover: float
+    missing: str
+
+
+class step(NamedTuple):
+    step_name: StepName
+    run_condition: bool
+    cmds: Tuple[str, ...]
+    log_message: str
+    timeout: float
+
+
+class test_result(NamedTuple):
+    setup_py_path: Path
+    returncode: int
+    output: str
+    runtime: float
+    timeout: bool
 
 
 def _get_site_packages_path(venv_path: Path) -> Optional[Path]:
@@ -188,13 +222,13 @@ def _analyze_coverage(
             )
 
         if sl[0] != "TOTAL":
-            stats[
-                f"suite.{module_path.name}_coverage.file.{module_path_str}"
-            ] = coverage_lines[module_path_str].cover
+            stats[f"suite.{module_path.name}_coverage.file.{module_path_str}"] = int(
+                coverage_lines[module_path_str].cover
+            )
         else:
-            stats[f"suite.{module_path.name}_coverage.total"] = coverage_lines[
-                module_path_str
-            ].cover
+            stats[f"suite.{module_path.name}_coverage.total"] = int(
+                coverage_lines[module_path_str].cover
+            )
 
     failed_output = "The following files did not meet coverage requirements:\n"
     failed_coverage = False
@@ -204,8 +238,8 @@ def _analyze_coverage(
             cover = coverage_lines[afile].cover
         except KeyError:
             err = (
-                "{} has not reported any coverage. Does the file exist? "
-                "Does it get ran during tests? Remove from setup config.".format(afile)
+                f"{afile} has not reported any coverage. Does the file exist? "
+                + "Does it get ran during tests? Remove from setup config."
             )
             keyerror_runtime = int(time() - test_run_start_time)
             return test_result(
@@ -218,12 +252,8 @@ def _analyze_coverage(
 
         if cover < cov_req:
             failed_coverage = True
-            failed_output += "  {}: {} < {} - Missing: {}\n".format(
-                afile,
-                coverage_lines[afile].cover,
-                cov_req,
-                coverage_lines[afile].missing,
-            )
+            cov_lines = coverage_lines[afile]
+            failed_output += f"  {afile}: {cov_lines.cover} < {cov_req} - Missing: {cov_lines.missing}\n"
 
     if failed_coverage:
         failed_cov_runtime = int(time() - test_run_start_time)
@@ -327,14 +357,14 @@ def _generate_pylint_cmd(
     if not config.get("run_pylint", False):
         return ()
 
-    py_files = set()  # type: Set[str]
+    py_files: Set[str] = set()
     find_py_files(py_files, module_dir)
 
     cmds = [str(pylint_exe)]
     pylint_config = module_dir / ".pylint"
     if pylint_config.exists():
         cmds.extend(["--rcfile", str(pylint_config)])
-    return (*cmds, *sorted(py_files))
+    return tuple([*cmds, *sorted(py_files)])
 
 
 def _generate_pyre_cmd(
@@ -352,7 +382,7 @@ def _generate_usort_cmd(
     if not config.get("run_usort", False):
         return ()
 
-    py_files = set()  # type: Set[str]
+    py_files: Set[str] = set()
     find_py_files(py_files, module_dir)
 
     return (str(usort_exe), "check", *sorted(py_files))
@@ -391,12 +421,15 @@ def _get_test_modules(
     )
     stats["total.setup_pys"] = len(all_setup_pys)
 
-    non_configured_modules = []  # type: List[Path]
-    test_modules = {}  # type: Dict[Path, Dict]
+    non_configured_modules: List[Path] = []
+    test_modules: Dict[Path, Dict] = {}
     for setup_py in all_setup_pys:
         disabled_err_msg = f"Not running {setup_py} as ptr is disabled via config"
-        # If a setup.cfg exists lets prefer it, if there is a [ptr] section
-        ptr_params = parse_setup_cfg(setup_py)
+        # If a pyproject.toml or setup.cfg exists lets prefer them
+        # Only if there is a [ptr] section
+        ptr_params = parse_pyproject_toml(setup_py)
+        if not ptr_params:
+            ptr_params = parse_setup_cfg(setup_py)
         if not ptr_params:
             ptr_params = _parse_setup_params(setup_py)
 
@@ -475,11 +508,8 @@ async def _progress_reporter(
 ) -> None:
     while queue.qsize() > 0:
         done_count = total_tests - queue.qsize()
-        LOG.info(
-            "{} / {} test suites ran ({}%)".format(
-                done_count, total_tests, int((done_count / total_tests) * 100)
-            )
-        )
+        done_pct = int((done_count / total_tests) * 100)
+        LOG.info(f"{done_count} / {total_tests} test suites ran ({done_pct}%)")
         await asyncio.sleep(progress_interval)
 
     LOG.debug("progress_reporter finished")
@@ -515,9 +545,7 @@ def _set_build_env(build_base_path: Optional[Path]) -> Dict[str, str]:
                 build_environ[var_name] = str(value)
     else:
         LOG.error(
-            "{} does not exist. Not add int PATH + INCLUDE Env variables".format(
-                build_base_path
-            )
+            f"{build_base_path} does not exist. Not add int PATH + INCLUDE Env variables"
         )
 
     return build_environ
@@ -526,6 +554,10 @@ def _set_build_env(build_base_path: Optional[Path]) -> Dict[str, str]:
 def _set_pip_mirror(
     venv_path: Path, mirror: str = CONFIG["ptr"]["pypi_url"], timeout: int = 2
 ) -> None:
+    if not venv_path.exists():
+        LOG.error(f"{venv_path} does not exist - So NOT writing out a pip.conf")
+        return
+
     pip_conf_path = venv_path / "pip.conf"
     with pip_conf_path.open("w", encoding="utf8") as pcfp:
         print(PIP_CONF_TEMPLATE.format(mirror, timeout), file=pcfp)
@@ -723,7 +755,7 @@ async def _test_runner(
         try:
             setup_py_path = queue.get_nowait()
         except asyncio.QueueEmpty:
-            LOG.debug("test_runner {} exiting".format(idx))
+            LOG.debug(f"test_runner {idx} exiting")
             return
 
         test_run_start_time = int(time())
@@ -807,6 +839,9 @@ def find_py_files(py_files: Set[str], base_dir: Path) -> None:
 def _recursive_find_files(
     files: Set[Path], base_dir: Path, exclude_patterns: Set[str], follow_symlinks: bool
 ) -> None:
+    if not base_dir.exists():
+        return
+
     dirs = [d for d in base_dir.iterdir() if d.is_dir()]
     files.update(
         {x for x in base_dir.iterdir() if x.is_file() and x.name == "setup.py"}
@@ -817,6 +852,9 @@ def _recursive_find_files(
 
         skip_dir = False
         for exclude_pattern in exclude_patterns:
+            if not exclude_pattern or exclude_pattern == ".":
+                LOG.error(f"Got a bad/empty exclude pattern: {exclude_pattern}")
+                continue
             if directory.match(exclude_pattern):
                 skip_dir = True
                 LOG.debug(
@@ -829,24 +867,42 @@ def _recursive_find_files(
 def find_setup_pys(
     base_path: Path, exclude_patterns: Set[str], follow_symlinks: bool = False
 ) -> Set[Path]:
-    setup_pys = set()  # type: Set[Path]
+    setup_pys: Set[Path] = set()
     _recursive_find_files(setup_pys, base_path, exclude_patterns, follow_symlinks)
     return setup_pys
 
 
+def parse_pyproject_toml(
+    setup_py: Path, tool_section: str = "tool", ptr_section: str = "ptr"
+) -> Dict[str, Any]:
+    ptr_params: Dict[str, Any] = {}
+    pyproject_toml_path = setup_py.parent / PYPROJECT_TOML
+    if not pyproject_toml_path.exists():
+        return ptr_params
+
+    with pyproject_toml_path.open("rb") as f:
+        pyproject_toml = tomllib.load(f)
+    ptr_params = pyproject_toml.get(tool_section, {}).get(ptr_section, {})
+
+    if not ptr_params:
+        LOG.info(f"{pyproject_toml} does not have a tool.ptr section")
+        return ptr_params
+
+    return ptr_params
+
+
 def parse_setup_cfg(setup_py: Path) -> Dict[str, Any]:
     req_cov_key_strip = "required_coverage_"
-    ptr_params = {}  # type: Dict[str, Any]
+    ptr_params: Dict[str, Any] = {}
     setup_cfg = setup_py.parent / "setup.cfg"
     if not setup_cfg.exists():
         return ptr_params
 
     cp = ConfigParser()
-    # Have configparser maintain key case - T484 error = callable
     cp.optionxform = str  # type: ignore
     cp.read(setup_cfg)
     if "ptr" not in cp:
-        LOG.info("{} does not have a ptr section")
+        LOG.info(f"{setup_cfg} does not have a ptr section")
         return ptr_params
 
     # Create a setup.py like ptr_params to return
@@ -877,7 +933,15 @@ def print_test_results(
     if not stats:
         stats = defaultdict(int)
 
+    # Ensure we always have 0 counters in stats JSON output
+    # Thus must have been working due to defaultdict + printing the keys
+    # Let us be more explicit
+    stats["total.fails"] = 0
+    stats["total.passes"] = 0
     stats["total.test_suites"] = len(test_results)
+    stats["total.timeouts"] = 0
+    if "total.disabled" not in stats:
+        stats["total.disabled"] = 0
 
     fail_output = ""
     for result in sorted(test_results):
@@ -886,8 +950,9 @@ def print_test_results(
                 stats["total.timeouts"] += 1
             else:
                 stats["total.fails"] += 1
-            fail_output += "{} (failed '{}' step):\n{}\n".format(
-                result.setup_py_path, StepName(result.returncode).name, result.output
+            fail_output += (
+                f"{result.setup_py_path} (failed '{StepName(result.returncode).name}' "
+                + f"step):\n{result.output}\n"
             )
         else:
             stats["total.passes"] += 1
@@ -896,25 +961,19 @@ def print_test_results(
     print(f"-- Summary (total time {total_time}s):\n")
     # TODO: Hardcode some workaround to ensure Windows always prints UTF8
     # https://github.com/facebookincubator/ptr/issues/34
-    print(
-        "✅ PASS: {}\n❌ FAIL: {}\n️⌛ TIMEOUT: {}\n🔒 DISABLED: {}\n💩 TOTAL: {}\n".format(
-            stats["total.passes"],
-            stats["total.fails"],
-            stats["total.timeouts"],
-            stats["total.disabled"],
-            stats["total.test_suites"],
-        )
-    )
-    if "total.setup_pys" in stats:
+    print(f"✅ PASS: {stats.get('total.passes', 0)}")
+    print(f"❌ FAIL: {stats.get('total.fails', 0)}")
+    print(f"⌛ TIMEOUT: {stats.get('total.timeouts', 0)}")
+    print(f"🔒 DISABLED: {stats.get('total.disabled', 0)}")
+    print(f"💩 TOTAL: {stats.get('total.test_suites', 0)}\n")
+    if "total.setup_pys" in stats and stats["total.setup_pys"] > 0:
         stats["pct.setup_py_ptr_enabled"] = int(
-            (stats["total.test_suites"] / stats["total.setup_pys"]) * 100
+            stats["total.test_suites"] / stats["total.setup_pys"] * 100
         )
         print(
-            "-- {} / {} ({}%) `setup.py`'s have `ptr` tests running\n".format(
-                stats["total.test_suites"],
-                stats["total.setup_pys"],
-                stats["pct.setup_py_ptr_enabled"],
-            )
+            f"-- {stats['total.test_suites']} / {stats['total.setup_pys']} "
+            + f"({stats['pct.setup_py_ptr_enabled']}%) `setup.py`'s have "
+            + "`ptr` tests running\n"
         )
     if fail_output:
         print("-- Failure Output --\n")
@@ -960,7 +1019,7 @@ async def run_tests(
     for test_setup_py in sorted(tests_to_run.keys()):
         await queue.put(test_setup_py)
 
-    test_results = []  # type: List[test_result]
+    test_results: List[test_result] = []
     consumers = [
         _test_runner(
             queue,
@@ -981,6 +1040,9 @@ async def run_tests(
         )
 
     LOG.debug("Starting to run tests")
+    if not consumers:
+        LOG.error("Got no _test_runner coros to gather. Exiting run_tests.")
+        return 254
     await asyncio.gather(*consumers)
 
     stats["runtime.all_tests"] = int(time() - tests_start_time)
@@ -993,7 +1055,7 @@ async def run_tests(
     else:
         LOG.info(f"Not removing venv @ {venv_path} due to CLI arguments")
 
-    return stats["total.fails"] + stats["total.timeouts"]
+    return stats.get("total.fails", 0) + stats.get("total.timeouts", 0)
 
 
 async def async_main(
@@ -1011,7 +1073,7 @@ async def async_main(
     error_on_warnings: bool,
     system_site_packages: bool,
 ) -> int:
-    stats = defaultdict(int)  # type: Dict[str, int]
+    stats: Dict[str, int] = defaultdict(int)
     tests_to_run = _get_test_modules(
         base_path, stats, run_disabled, print_non_configured
     )
@@ -1025,7 +1087,7 @@ async def async_main(
         return 0
 
     try:
-        venv_path = Path(venv)  # type: Optional[Path]
+        venv_path: Optional[Path] = Path(venv)
         if venv_path and not venv_path.exists():
             LOG.error(f"{venv_path} venv does not exist. Please correct!")
             return 2
@@ -1080,8 +1142,8 @@ def main() -> None:
         "-m",
         "--mirror",
         default=CONFIG["ptr"]["pypi_url"],
-        help="URL for pip to use for Simple API [Default: {}]".format(
-            CONFIG["ptr"]["pypi_url"]
+        help=(
+            f"URL for pip to use for Simple API [Default: {CONFIG['ptr']['pypi_url']}]"
         ),
     )
     parser.add_argument(
@@ -1127,30 +1189,33 @@ def main() -> None:
     _handle_debug(args.debug)
 
     LOG.info(f"Starting {sys.argv[0]}")
-    loop = asyncio.get_event_loop()
-    try:
-        sys.exit(
-            loop.run_until_complete(
-                async_main(
-                    args.atonce,
-                    _validate_base_dir(args.base_dir),
-                    args.mirror,
-                    args.progress_interval,
-                    args.venv,
-                    args.keep_venv,
-                    args.print_cov,
-                    args.print_non_configured,
-                    args.run_disabled,
-                    args.stats_file,
-                    args.venv_timeout,
-                    args.error_on_warnings,
-                    args.system_site_packages,
-                )
-            )
-        )
-    finally:
-        loop.close()
+    main_coro = async_main(
+        args.atonce,
+        _validate_base_dir(args.base_dir),
+        args.mirror,
+        args.progress_interval,
+        args.venv,
+        args.keep_venv,
+        args.print_cov,
+        args.print_non_configured,
+        args.run_disabled,
+        args.stats_file,
+        args.venv_timeout,
+        args.error_on_warnings,
+        args.system_site_packages,
+    )
+    # This is gated to >= 3.8 for unittests
+    # Once we're >= 3.7 tests could be refactored so we can use
+    # asyncio.run in 3.7
+    if getattr(asyncio, "run", None) and PY_38_OR_GREATER:
+        sys.exit(asyncio.run(main_coro))
+    else:
+        loop = asyncio.get_event_loop()
+        try:
+            sys.exit(loop.run_until_complete(main_coro))
+        finally:
+            loop.close()
 
 
-if __name__ == "__main__":
-    main()  # pragma: no cover
+if __name__ == "__main__":  # pragma: no cover
+    main()
