@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright © Meta Platforms, Inc
+# Copyright © Meta Platforms, Inc. and affiliates
 
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -13,8 +13,8 @@ from os import environ
 from pathlib import Path
 from shutil import rmtree
 from subprocess import CalledProcessError
-from tempfile import TemporaryDirectory, gettempdir
-from typing import (  # noqa: F401 # pylint: disable=unused-import
+from tempfile import gettempdir, TemporaryDirectory
+from typing import (  # noqa: F401  # pylint: disable=unused-import
     Any,
     Dict,
     List,
@@ -27,6 +27,9 @@ from unittest.mock import Mock, patch
 import ptr
 import ptr_tests_fixtures
 
+# Set a global for our file encoding - pylint recommendation
+FILE_ENCODING = "utf8"
+
 # Turn off logging for unit tests - Comment out to enable
 ptr.LOG = Mock()
 # Hacky global for a monkey patch of asyncio.qsize()
@@ -269,7 +272,7 @@ class TestPtr(unittest.TestCase):
             build_path = td_path / "build-arm"
             cooper_path = td_path / "cooper"
 
-            touch_files(*(adir / "setup.py" for adir in {build_path, cooper_path}))
+            touch_files(*(adir / "setup.py" for adir in (build_path, cooper_path)))
 
             setup_pys = ptr.find_setup_pys(td_path, {"build*"})
             self.assertEqual(len(setup_pys), 1)
@@ -371,10 +374,17 @@ class TestPtr(unittest.TestCase):
             lib_path.mkdir()
             self.assertIsNone(ptr._get_site_packages_path(lib_path.parent))
 
+    # Patch parsing except setup.py to keep coverage up
+    @patch("ptr.parse_setup_cfg")
+    @patch("ptr.parse_pyproject_toml")
     @patch("ptr.print")  # noqa
-    def test_get_test_modules(self, mock_print: Mock) -> None:
+    def test_get_test_modules(
+        self, mock_print: Mock, mock_pyproject: Mock, mock_setup_cfg: Mock
+    ) -> None:
+        mock_pyproject.return_value = {}
+        mock_setup_cfg.return_value = {}
         base_path = Path(__file__).parent
-        stats = defaultdict(int)  # type: Dict[str, int]
+        stats: Dict[str, int] = defaultdict(int)
         test_modules = ptr._get_test_modules(base_path, stats, True, True)
         self.assertEqual(
             test_modules[base_path / "setup.py"],
@@ -422,12 +432,29 @@ class TestPtr(unittest.TestCase):
             with self.assertRaises(SystemExit):
                 ptr.main()
 
+    def test_parse_pyproject_toml(self) -> None:
+        tmp_dir = Path(gettempdir())
+        pyproject_toml = tmp_dir / ptr.PYPROJECT_TOML
+        setup_py = tmp_dir / "setup.py"
+
+        with pyproject_toml.open("w", encoding=FILE_ENCODING) as pcp:
+            pcp.write(ptr_tests_fixtures.SAMPLE_PYPROJECT)
+
+        # No pyproject.toml file exists
+        self.assertEqual(ptr.parse_pyproject_toml(setup_py.parent), {})
+        # No ptr section
+        self.assertEqual(ptr.parse_pyproject_toml(setup_py, "not_a_tool"), {})
+        # Everything works
+        self.assertEqual(
+            ptr.parse_pyproject_toml(setup_py), ptr_tests_fixtures.EXPECTED_TEST_PARAMS
+        )
+
     def test_parse_setup_cfg(self) -> None:
         tmp_dir = Path(gettempdir())
         setup_cfg = tmp_dir / "setup.cfg"
         setup_py = tmp_dir / "setup.py"
 
-        with setup_cfg.open("w") as scp:
+        with setup_cfg.open("w", encoding=FILE_ENCODING) as scp:
             scp.write(ptr_tests_fixtures.SAMPLE_SETUP_CFG)
 
         self.assertEqual(
@@ -478,7 +505,7 @@ class TestPtr(unittest.TestCase):
             fake_venv_path = Path(td)
             pip_conf_path = fake_venv_path / "pip.conf"
             ptr._set_pip_mirror(fake_venv_path)
-            with pip_conf_path.open("r") as pcfp:
+            with pip_conf_path.open("r", encoding=FILE_ENCODING) as pcfp:
                 conf_file = pcfp.read()
             self.assertTrue("[global]" in conf_file)
             self.assertTrue("/simple" in conf_file)
@@ -489,7 +516,7 @@ class TestPtr(unittest.TestCase):
         with TemporaryDirectory() as td:
             td_path = Path(td)
             setup_py_path = td_path / "setup.py"
-            with setup_py_path.open("w") as spfp:
+            with setup_py_path.open("w", encoding=FILE_ENCODING) as spfp:
                 print(ptr_tests_fixtures.SAMPLE_SETUP_PY, file=spfp)
 
             queue.put_nowait(setup_py_path)
